@@ -3,24 +3,37 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 "use strict";
 
-const PAGE = "http://mochi.test:8888/browser/browser/components/extensions/test/browser/context.html";
+const { GlobalManager } = ChromeUtils.import(
+  "resource://gre/modules/Extension.jsm",
+  null
+);
+const { ExtensionPermissions } = ChromeUtils.import(
+  "resource://gre/modules/ExtensionPermissions.jsm"
+);
+
+const PAGE =
+  "http://mochi.test:8888/browser/browser/components/extensions/test/browser/context.html";
 const PAGE_BASE = PAGE.replace("context.html", "");
 const PAGE_HOST_PATTERN = "http://mochi.test/*";
 
+const EXPECT_TARGET_ELEMENT = 13337;
+
 async function grantOptionalPermission(extension, permissions) {
-  const {GlobalManager} = ChromeUtils.import("resource://gre/modules/Extension.jsm", {});
-  const {ExtensionPermissions} = ChromeUtils.import("resource://gre/modules/ExtensionPermissions.jsm", {});
   let ext = GlobalManager.extensionMap.get(extension.id);
-  return ExtensionPermissions.add(ext, permissions);
+  return ExtensionPermissions.add(extension.id, permissions, ext);
 }
 
 // Registers a context menu using menus.create(menuCreateParams) and checks
 // whether the menus.onShown and menus.onHidden events are fired as expected.
 // doOpenMenu must open the menu and its returned promise must resolve after the
 // menu is shown. Similarly, doCloseMenu must hide the menu.
-async function testShowHideEvent({menuCreateParams, doOpenMenu, doCloseMenu,
-                                  expectedShownEvent,
-                                  expectedShownEventWithPermissions = null}) {
+async function testShowHideEvent({
+  menuCreateParams,
+  doOpenMenu,
+  doCloseMenu,
+  expectedShownEvent,
+  expectedShownEventWithPermissions = null,
+}) {
   async function background() {
     function awaitMessage(expectedId) {
       return new Promise(resolve => {
@@ -40,14 +53,26 @@ async function testShowHideEvent({menuCreateParams, doOpenMenu, doCloseMenu,
     }
 
     let menuCreateParams = await awaitMessage("create-params");
-    const [tab] = await browser.tabs.query({active: true, currentWindow: true});
+    const [tab] = await browser.tabs.query({
+      active: true,
+      currentWindow: true,
+    });
 
     let shownEvents = [];
     let hiddenEvents = [];
 
     browser.menus.onShown.addListener((...args) => {
+      if (args[0].targetElementId) {
+        // In this test, we aren't interested in the exact value,
+        // only in whether it is set or not.
+        args[0].targetElementId = 13337; // = EXPECT_TARGET_ELEMENT
+      }
       shownEvents.push(args[0]);
-      browser.test.assertEq(tab.id, args[1].id, "expected tab");
+      if (menuCreateParams.title.includes("TEST_EXPECT_NO_TAB")) {
+        browser.test.assertEq(undefined, args[1], "expect no tab");
+      } else {
+        browser.test.assertEq(tab.id, args[1].id, "expected tab");
+      }
       browser.test.assertEq(2, args.length, "expected number of onShown args");
     });
     browser.menus.onHidden.addListener(event => hiddenEvents.push(event));
@@ -83,10 +108,15 @@ async function testShowHideEvent({menuCreateParams, doOpenMenu, doCloseMenu,
     background,
     manifest: {
       page_action: {},
-      browser_action: {},
+      browser_action: {
+        default_popup: "popup.html",
+      },
       permissions: ["menus"],
       optional_permissions: [PAGE_HOST_PATTERN],
     },
+    files: {
+      "popup.html": `<!DOCTYPE html><meta charset="utf-8">Popup body`,
+    },
   });
   await extension.startup();
   extension.sendMessage("create-params", menuCreateParams);
@@ -100,7 +130,7 @@ async function testShowHideEvent({menuCreateParams, doOpenMenu, doCloseMenu,
   expectedShownEvent.menuIds = [menuId];
   Assert.deepEqual(shownEvent, expectedShownEvent, "expected onShown info");
 
-  await doCloseMenu();
+  await doCloseMenu(extension);
   extension.sendMessage("assert-menu-hidden");
   let hiddenEvent = await extension.awaitMessage("onHidden-event-data");
   is(hiddenEvent, undefined, "expected no event data for onHidden event");
@@ -114,13 +144,16 @@ async function testShowHideEvent({menuCreateParams, doOpenMenu, doCloseMenu,
     await doOpenMenu(extension);
     extension.sendMessage("optional-menu-shown-with-permissions");
     let shownEvent2 = await extension.awaitMessage("onShown-event-data2");
-    Assert.deepEqual(shownEvent2, expectedShownEventWithPermissions,
-                     "expected onShown info when host permissions are enabled");
-    await doCloseMenu();
+    Assert.deepEqual(
+      shownEvent2,
+      expectedShownEventWithPermissions,
+      "expected onShown info when host permissions are enabled"
+    );
+    await doCloseMenu(extension);
   }
 
   await extension.unload();
-  await BrowserTestUtils.removeTab(tab);
+  BrowserTestUtils.removeTab(tab);
 }
 
 // Make sure that we won't trigger onShown when extensions cannot add menus.
@@ -131,8 +164,11 @@ add_task(async function test_no_show_hide_for_unsupported_menu() {
       browser.menus.onShown.addListener(data => events.push(data));
       browser.menus.onHidden.addListener(() => events.push("onHidden"));
       browser.test.onMessage.addListener(() => {
-        browser.test.assertEq("[]", JSON.stringify(events),
-                              "Should not have any events when the context is unsupported.");
+        browser.test.assertEq(
+          "[]",
+          JSON.stringify(events),
+          "Should not have any events when the context is unsupported."
+        );
         browser.test.notifyPass("done listening to menu events");
       });
     },
@@ -183,6 +219,7 @@ add_task(async function test_show_hide_without_menu_item() {
     },
     expectedShownEvent: {
       contexts: ["page", "all"],
+      viewType: "tab",
       editable: false,
       frameId: 0,
     },
@@ -200,13 +237,20 @@ add_task(async function test_show_hide_without_menu_item() {
   let events = await extension.awaitMessage("events from menuless extension");
   is(events.length, 2, "expect two events");
   is(events[1], "onHidden", "last event should be onHidden");
-  Assert.deepEqual(events[0], {
-    menuIds: [],
-    contexts: ["page", "all"],
-    editable: false,
-    pageUrl: PAGE,
-    frameId: 0,
-  }, "expected onShown info from menuless extension");
+  ok(events[0].targetElementId, "info.targetElementId must be set in onShown");
+  delete events[0].targetElementId;
+  Assert.deepEqual(
+    events[0],
+    {
+      menuIds: [],
+      contexts: ["page", "all"],
+      viewType: "tab",
+      editable: false,
+      pageUrl: PAGE,
+      frameId: 0,
+    },
+    "expected onShown info from menuless extension"
+  );
   await extension.unload();
 });
 
@@ -218,10 +262,12 @@ add_task(async function test_show_hide_pageAction() {
     },
     expectedShownEvent: {
       contexts: ["page_action", "all"],
+      viewType: undefined,
       editable: false,
     },
     expectedShownEventWithPermissions: {
       contexts: ["page_action", "all"],
+      viewType: undefined,
       editable: false,
       pageUrl: PAGE,
     },
@@ -242,10 +288,12 @@ add_task(async function test_show_hide_browserAction() {
     },
     expectedShownEvent: {
       contexts: ["browser_action", "all"],
+      viewType: undefined,
       editable: false,
     },
     expectedShownEventWithPermissions: {
       contexts: ["browser_action", "all"],
+      viewType: undefined,
       editable: false,
       pageUrl: PAGE,
     },
@@ -258,6 +306,45 @@ add_task(async function test_show_hide_browserAction() {
   });
 });
 
+add_task(async function test_show_hide_browserAction_popup() {
+  let popupUrl;
+  await testShowHideEvent({
+    menuCreateParams: {
+      title: "browserAction popup - TEST_EXPECT_NO_TAB",
+      contexts: ["all", "browser_action"],
+    },
+    expectedShownEvent: {
+      contexts: ["page", "all"],
+      viewType: "popup",
+      frameId: 0,
+      editable: false,
+      get pageUrl() {
+        return popupUrl;
+      },
+      targetElementId: EXPECT_TARGET_ELEMENT,
+    },
+    expectedShownEventWithPermissions: {
+      contexts: ["page", "all"],
+      viewType: "popup",
+      frameId: 0,
+      editable: false,
+      get pageUrl() {
+        return popupUrl;
+      },
+      targetElementId: EXPECT_TARGET_ELEMENT,
+    },
+    async doOpenMenu(extension) {
+      popupUrl = `moz-extension://${extension.uuid}/popup.html`;
+      await clickBrowserAction(extension);
+      await openContextMenuInPopup(extension);
+    },
+    async doCloseMenu(extension) {
+      await closeExtensionContextMenu();
+      await closeBrowserAction(extension);
+    },
+  });
+});
+
 add_task(async function test_show_hide_tab() {
   await testShowHideEvent({
     menuCreateParams: {
@@ -266,10 +353,12 @@ add_task(async function test_show_hide_tab() {
     },
     expectedShownEvent: {
       contexts: ["tab"],
+      viewType: undefined,
       editable: false,
     },
     expectedShownEventWithPermissions: {
       contexts: ["tab"],
+      viewType: undefined,
       editable: false,
       pageUrl: PAGE,
     },
@@ -290,10 +379,12 @@ add_task(async function test_show_hide_tools_menu() {
     },
     expectedShownEvent: {
       contexts: ["tools_menu"],
+      viewType: undefined,
       editable: false,
     },
     expectedShownEventWithPermissions: {
       contexts: ["tools_menu"],
+      viewType: undefined,
       editable: false,
       pageUrl: PAGE,
     },
@@ -314,14 +405,17 @@ add_task(async function test_show_hide_page() {
     },
     expectedShownEvent: {
       contexts: ["page", "all"],
+      viewType: "tab",
       editable: false,
       frameId: 0,
     },
     expectedShownEventWithPermissions: {
       contexts: ["page", "all"],
+      viewType: "tab",
       editable: false,
       pageUrl: PAGE,
       frameId: 0,
+      targetElementId: EXPECT_TARGET_ELEMENT,
     },
     async doOpenMenu() {
       await openContextMenu("body");
@@ -342,22 +436,37 @@ add_task(async function test_show_hide_frame() {
     },
     expectedShownEvent: {
       contexts: ["frame", "all"],
+      viewType: "tab",
       editable: false,
-      get frameId() { return frameId; },
+      get frameId() {
+        return frameId;
+      },
     },
     expectedShownEventWithPermissions: {
       contexts: ["frame", "all"],
+      viewType: "tab",
       editable: false,
-      get frameId() { return frameId; },
+      get frameId() {
+        return frameId;
+      },
       pageUrl: PAGE,
       frameUrl: PAGE_BASE + "context_frame.html",
+      targetElementId: EXPECT_TARGET_ELEMENT,
     },
     async doOpenMenu() {
-      frameId = await ContentTask.spawn(gBrowser.selectedBrowser, {}, function() {
-        let {contentWindow} = content.document.getElementById("frame");
-        return WebNavigationFrames.getFrameId(contentWindow);
-      });
-      await openContextMenuInFrame("frame");
+      frameId = await ContentTask.spawn(
+        gBrowser.selectedBrowser,
+        {},
+        function() {
+          const { WebNavigationFrames } = ChromeUtils.import(
+            "resource://gre/modules/WebNavigationFrames.jsm"
+          );
+
+          let { contentWindow } = content.document.getElementById("frame");
+          return WebNavigationFrames.getFrameId(contentWindow);
+        }
+      );
+      await openContextMenuInFrame("#frame");
     },
     async doCloseMenu() {
       await closeExtensionContextMenu();
@@ -373,14 +482,17 @@ add_task(async function test_show_hide_password() {
     },
     expectedShownEvent: {
       contexts: ["editable", "password", "all"],
+      viewType: "tab",
       editable: true,
       frameId: 0,
     },
     expectedShownEventWithPermissions: {
       contexts: ["editable", "password", "all"],
+      viewType: "tab",
       editable: true,
       frameId: 0,
       pageUrl: PAGE,
+      targetElementId: EXPECT_TARGET_ELEMENT,
     },
     async doOpenMenu() {
       await openContextMenu("#password");
@@ -399,16 +511,19 @@ add_task(async function test_show_hide_link() {
     },
     expectedShownEvent: {
       contexts: ["link", "all"],
+      viewType: "tab",
       editable: false,
       frameId: 0,
     },
     expectedShownEventWithPermissions: {
       contexts: ["link", "all"],
+      viewType: "tab",
       editable: false,
       frameId: 0,
       linkText: "Some link",
       linkUrl: PAGE_BASE + "some-link",
       pageUrl: PAGE,
+      targetElementId: EXPECT_TARGET_ELEMENT,
     },
     async doOpenMenu() {
       await openContextMenu("#link1");
@@ -427,12 +542,14 @@ add_task(async function test_show_hide_image_link() {
     },
     expectedShownEvent: {
       contexts: ["image", "link", "all"],
+      viewType: "tab",
       mediaType: "image",
       editable: false,
       frameId: 0,
     },
     expectedShownEventWithPermissions: {
       contexts: ["image", "link", "all"],
+      viewType: "tab",
       mediaType: "image",
       editable: false,
       frameId: 0,
@@ -441,6 +558,7 @@ add_task(async function test_show_hide_image_link() {
       linkUrl: PAGE_BASE + "image-around-some-link",
       srcUrl: PAGE_BASE + "ctxmenu-image.png",
       pageUrl: PAGE,
+      targetElementId: EXPECT_TARGET_ELEMENT,
     },
     async doOpenMenu() {
       await openContextMenu("#img-wrapped-in-link");
@@ -460,24 +578,33 @@ add_task(async function test_show_hide_editable_selection() {
     },
     expectedShownEvent: {
       contexts: ["editable", "selection", "all"],
+      viewType: "tab",
       editable: true,
       frameId: 0,
     },
     expectedShownEventWithPermissions: {
       contexts: ["editable", "selection", "all"],
+      viewType: "tab",
       editable: true,
       frameId: 0,
       pageUrl: PAGE,
-      get selectionText() { return selectionText; },
+      get selectionText() {
+        return selectionText;
+      },
+      targetElementId: EXPECT_TARGET_ELEMENT,
     },
     async doOpenMenu() {
       // Select lots of text in the test page before opening the menu.
-      selectionText = await ContentTask.spawn(gBrowser.selectedBrowser, {}, function() {
-        let node = content.document.getElementById("editabletext");
-        node.select();
-        node.focus();
-        return node.value;
-      });
+      selectionText = await ContentTask.spawn(
+        gBrowser.selectedBrowser,
+        {},
+        function() {
+          let node = content.document.getElementById("editabletext");
+          node.select();
+          node.focus();
+          return node.value;
+        }
+      );
 
       await openContextMenu("#editabletext");
     },
@@ -487,5 +614,88 @@ add_task(async function test_show_hide_editable_selection() {
   });
 });
 
-// TODO(robwu): Add test coverage for contexts audio, video (bug 1398542).
+add_task(async function test_show_hide_video() {
+  const VIDEO_URL = "data:video/webm,xxx";
+  await testShowHideEvent({
+    menuCreateParams: {
+      title: "video item",
+      contexts: ["video"],
+    },
+    expectedShownEvent: {
+      contexts: ["video", "all"],
+      viewType: "tab",
+      mediaType: "video",
+      editable: false,
+      frameId: 0,
+    },
+    expectedShownEventWithPermissions: {
+      contexts: ["video", "all"],
+      viewType: "tab",
+      mediaType: "video",
+      editable: false,
+      frameId: 0,
+      srcUrl: VIDEO_URL,
+      pageUrl: PAGE,
+      targetElementId: EXPECT_TARGET_ELEMENT,
+    },
+    async doOpenMenu() {
+      await ContentTask.spawn(gBrowser.selectedBrowser, VIDEO_URL, function(
+        VIDEO_URL
+      ) {
+        let video = content.document.createElement("video");
+        video.controls = true;
+        video.src = VIDEO_URL;
+        content.document.body.appendChild(video);
+        video.focus();
+      });
 
+      await openContextMenu("video");
+    },
+    async doCloseMenu() {
+      await closeExtensionContextMenu();
+    },
+  });
+});
+
+add_task(async function test_show_hide_audio() {
+  const AUDIO_URL = "data:audio/ogg,xxx";
+  await testShowHideEvent({
+    menuCreateParams: {
+      title: "audio item",
+      contexts: ["audio"],
+    },
+    expectedShownEvent: {
+      contexts: ["audio", "all"],
+      viewType: "tab",
+      mediaType: "audio",
+      editable: false,
+      frameId: 0,
+    },
+    expectedShownEventWithPermissions: {
+      contexts: ["audio", "all"],
+      viewType: "tab",
+      mediaType: "audio",
+      editable: false,
+      frameId: 0,
+      srcUrl: AUDIO_URL,
+      pageUrl: PAGE,
+      targetElementId: EXPECT_TARGET_ELEMENT,
+    },
+    async doOpenMenu() {
+      await ContentTask.spawn(gBrowser.selectedBrowser, AUDIO_URL, function(
+        AUDIO_URL
+      ) {
+        let audio = content.document.createElement("audio");
+        audio.controls = true;
+        audio.src = AUDIO_URL;
+        content.document.body.appendChild(audio);
+        audio.focus();
+      });
+
+      await openContextMenu("audio");
+    },
+    async doCloseMenu() {
+      await closeExtensionContextMenu();
+    },
+  });
+});
