@@ -6,23 +6,23 @@
  * Some common, generic functions
  */
 
-ChromeUtils.import("resource:///modules/errUtils.js");
-ChromeUtils.import("resource://gre/modules/Services.jsm");
-/* eslint-disable spaced-comment */
+/* import-globals-from emailWizard.js */
 
+var { logException } = ChromeUtils.import("resource:///modules/errUtils.js");
+var { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
 
-/////////////////////////////////////////
+// --------------------------
 // Low level, basic functions
 
-function assert(test, errorMsg)
-{
-  if (!test)
-    throw new NotReached(errorMsg ? errorMsg :
-          "Programming bug. Assertion failed, see log.");
+function assert(test, errorMsg) {
+  if (!test) {
+    throw new NotReached(
+      errorMsg ? errorMsg : "Programming bug. Assertion failed, see log."
+    );
+  }
 }
 
-function makeCallback(obj, func)
-{
+function makeCallback(obj, func) {
   return func.bind(obj);
 }
 
@@ -35,8 +35,7 @@ function makeCallback(obj, func)
  *
  * @see |TimeoutAbortable|
  */
-function runAsync(func)
-{
+function runAsync(func) {
   return setTimeout(func, 0);
 }
 
@@ -44,8 +43,7 @@ function runAsync(func)
  * @param uriStr {String}
  * @result {nsIURI}
  */
-function makeNSIURI(uriStr)
-{
+function makeNSIURI(uriStr) {
   return Services.io.newURI(uriStr);
 }
 
@@ -55,27 +53,34 @@ function makeNSIURI(uriStr)
  * @param uri {nsIURI}   what you want to read
  * @return {Array of String}   the contents of the file, one string per line
  */
-function readURLasUTF8(uri)
-{
+function readURLasUTF8(uri) {
   assert(uri instanceof Ci.nsIURI, "uri must be an nsIURI");
   try {
-    let chan = Services.io.newChannelFromURI2(uri,
-                                              null,
-                                              Services.scriptSecurityManager.getSystemPrincipal(),
-                                              null,
-                                              Ci.nsILoadInfo.SEC_ALLOW_CROSS_ORIGIN_DATA_IS_NULL,
-                                              Ci.nsIContentPolicy.TYPE_OTHER);
-    let is = Cc["@mozilla.org/intl/converter-input-stream;1"]
-             .createInstance(Ci.nsIConverterInputStream);
-    is.init(chan.open(), "UTF-8", 1024,
-            Ci.nsIConverterInputStream.DEFAULT_REPLACEMENT_CHARACTER);
+    let chan = Services.io.newChannelFromURI(
+      uri,
+      null,
+      Services.scriptSecurityManager.getSystemPrincipal(),
+      null,
+      Ci.nsILoadInfo.SEC_ALLOW_CROSS_ORIGIN_DATA_IS_NULL,
+      Ci.nsIContentPolicy.TYPE_OTHER
+    );
+    let is = Cc["@mozilla.org/intl/converter-input-stream;1"].createInstance(
+      Ci.nsIConverterInputStream
+    );
+    is.init(
+      chan.open(),
+      "UTF-8",
+      1024,
+      Ci.nsIConverterInputStream.DEFAULT_REPLACEMENT_CHARACTER
+    );
 
     let content = "";
-    let strOut = new Object();
+    let strOut = {};
     try {
-      while (is.readString(1024, strOut) != 0)
+      while (is.readString(1024, strOut) != 0) {
         content += strOut.value;
-    // catch in outer try/catch
+      }
+      // catch in outer try/catch
     } finally {
       is.close();
     }
@@ -99,56 +104,58 @@ function readURLasUTF8(uri)
  * @param content {String} one long string with the whole file
  * @return {Array of String} one string per line (no linebreaks)
  */
-function splitLines(content)
-{
+function splitLines(content) {
   content = content.replace("\r\n", "\n");
   content = content.replace("\r", "\n");
   return content.split("\n");
 }
 
+/**
+ * Returns only one element for each result of testFunc.
+ * First element for each result wins.
+ * @param testFunc {Function(arrayElement)}
+ * @returns new array {Array}
+ */
+Array.prototype.unique = function(testFunc) {
+  return this.reduce((found, nextEl) => {
+    if (!found.some(prevEl => testFunc(prevEl) == testFunc(nextEl))) {
+      found.push(nextEl);
+    }
+    return found;
+  }, []);
+};
+
 /**
  * @param bundleURI {String}   chrome URL to properties file
  * @return nsIStringBundle
  */
-function getStringBundle(bundleURI)
-{
+function getStringBundle(bundleURI) {
   try {
     return Services.strings.createBundle(bundleURI);
   } catch (e) {
-    throw new Exception("Failed to get stringbundle URI <" + bundleURI +
-                        ">. Error: " + e);
+    throw new Exception(
+      "Failed to get stringbundle URI <" + bundleURI + ">. Error: " + e
+    );
   }
 }
 
-
-/////////////////////////////////////////
+// ---------
 // Exception
 
-function Exception(msg)
-{
+function Exception(msg) {
   this._message = msg;
-
-  // get stack
-  try {
-    not.found.here += 1; // force a native exception ...
-  } catch (e) {
-    this.stack = e.stack; // ... to get the current stack
-  }
+  this.stack = Components.stack.formattedStack;
 }
-Exception.prototype =
-{
-  get message()
-  {
+Exception.prototype = {
+  get message() {
     return this._message;
   },
-  toString : function()
-  {
+  toString() {
     return this._message;
-  }
-}
+  },
+};
 
-function NotReached(msg)
-{
+function NotReached(msg) {
   Exception.call(this, msg); // call super constructor
   logException(this);
 }
@@ -156,8 +163,7 @@ function NotReached(msg)
 NotReached.prototype = Object.create(Exception.prototype);
 NotReached.prototype.constructor = NotReached;
 
-
-/////////////////////////////////////////
+// ---------
 // Abortable
 
 /**
@@ -165,14 +171,10 @@ NotReached.prototype.constructor = NotReached;
  * The async function will return an object of this type (a subtype)
  * and you can call cancel() when you feel like killing the function.
  */
-function Abortable()
-{
-}
-Abortable.prototype =
-{
-  cancel(e) {
-  }
-}
+function Abortable() {}
+Abortable.prototype = {
+  cancel(e) {},
+};
 
 function CancelledException(msg) {
   Exception.call(this, msg);
@@ -180,40 +182,71 @@ function CancelledException(msg) {
 CancelledException.prototype = Object.create(Exception.prototype);
 CancelledException.prototype.constructor = CancelledException;
 
+/**
+ * Utility implementation, for waiting for a promise to resolve,
+ * but allowing its result to be cancelled.
+ */
+function PromiseAbortable(promise, successCallback, errorCallback) {
+  Abortable.call(this); // call super constructor
+  let complete = false;
+  this.cancel = function(e) {
+    if (!complete) {
+      complete = true;
+      errorCallback(e || new CancelledException());
+    }
+  };
+  promise
+    .then(function(result) {
+      if (!complete) {
+        successCallback(result);
+        complete = true;
+      }
+    })
+    .catch(function(e) {
+      if (!complete) {
+        complete = true;
+        errorCallback(e);
+      }
+    });
+}
+PromiseAbortable.prototype = Object.create(Abortable.prototype);
+PromiseAbortable.prototype.constructor = PromiseAbortable;
+
 /**
  * Utility implementation, for allowing to abort a setTimeout.
  * Use like: return new TimeoutAbortable(setTimeout(function(){ ... }, 0));
  * @param setTimeoutID {Integer}  Return value of setTimeout()
  */
-function TimeoutAbortable(setTimeoutID)
-{
+function TimeoutAbortable(setTimeoutID) {
   Abortable.call(this); // call super constructor
   this._id = setTimeoutID;
 }
 TimeoutAbortable.prototype = Object.create(Abortable.prototype);
 TimeoutAbortable.prototype.constructor = TimeoutAbortable;
-TimeoutAbortable.prototype.cancel = function() { clearTimeout(this._id); }
+TimeoutAbortable.prototype.cancel = function() {
+  clearTimeout(this._id);
+};
 
 /**
  * Utility implementation, for allowing to abort a setTimeout.
  * Use like: return new TimeoutAbortable(setTimeout(function(){ ... }, 0));
  * @param setIntervalID {Integer}  Return value of setInterval()
  */
-function IntervalAbortable(setIntervalID)
-{
+function IntervalAbortable(setIntervalID) {
   Abortable.call(this); // call super constructor
   this._id = setIntervalID;
 }
 IntervalAbortable.prototype = Object.create(Abortable.prototype);
 IntervalAbortable.prototype.constructor = IntervalAbortable;
-IntervalAbortable.prototype.cancel = function() { clearInterval(this._id); }
+IntervalAbortable.prototype.cancel = function() {
+  clearInterval(this._id);
+};
 
 /**
  * Allows you to make several network calls,
  * but return only one |Abortable| object.
  */
-function SuccessiveAbortable()
-{
+function SuccessiveAbortable() {
   Abortable.call(this); // call super constructor
   this._current = null;
 }
@@ -222,18 +255,19 @@ SuccessiveAbortable.prototype = {
   get current() {
     return this._current;
   },
-  set current(abortable)
-  {
-    assert(abortable instanceof Abortable || abortable == null,
-        "need an Abortable object (or null)");
+  set current(abortable) {
+    assert(
+      abortable instanceof Abortable || abortable == null,
+      "need an Abortable object (or null)"
+    );
     this._current = abortable;
   },
   cancel(e) {
     if (this._current) {
       this._current.cancel(e);
     }
-  }
-}
+  },
+};
 
 /**
  * Allows you to make several network calls in parallel.
@@ -268,7 +302,7 @@ ParallelAbortable.prototype = {
    * @param {Function({ParallelCall} call)} func
    */
   addOneFinishedObserver(func) {
-    assert(typeof(func) == "function");
+    assert(typeof func == "function");
     this._finishedObservers.push(func);
   },
   /**
@@ -281,7 +315,7 @@ ParallelAbortable.prototype = {
    *   )} func
    */
   addAllFinishedObserver(func) {
-    assert(typeof(func) == "function");
+    assert(typeof func == "function");
     this.addOneFinishedObserver(() => {
       if (this._calls.some(call => !call.finished)) {
         return;
@@ -341,8 +375,16 @@ ParallelCall.prototype = {
    */
   successCallback() {
     return result => {
-      ddump("call " + this.position + " took " + (Date.now() - this._time) + "ms and succeeded" +
-          (this.callerAbortable && this.callerAbortable._url ? " at <" + this.callerAbortable._url + ">" : ""));
+      ddump(
+        "call " +
+          this.position +
+          " took " +
+          (Date.now() - this._time) +
+          "ms and succeeded" +
+          (this.callerAbortable && this.callerAbortable._url
+            ? " at <" + this.callerAbortable._url + ">"
+            : "")
+      );
       this.result = result;
       this.finished = true;
       this.succeeded = true;
@@ -356,10 +398,20 @@ ParallelCall.prototype = {
    */
   errorCallback() {
     return e => {
-      ddump("call " + this.position + " took " + (Date.now() - this._time) +
-          "ms and failed with " + (typeof(e.code) == "number" ? e.code + " " : "") +
-          (e.toString() ? e.toString() : "unknown error, probably no host connection") +
-          (this.callerAbortable && this.callerAbortable._url ? " at <" + this.callerAbortable._url + ">" : ""));
+      ddump(
+        "call " +
+          this.position +
+          " took " +
+          (Date.now() - this._time) +
+          "ms and failed with " +
+          (typeof e.code == "number" ? e.code + " " : "") +
+          (e.toString()
+            ? e.toString()
+            : "unknown error, probably no host connection") +
+          (this.callerAbortable && this.callerAbortable._url
+            ? " at <" + this.callerAbortable._url + ">"
+            : "")
+      );
       this.e = e;
       this.finished = true;
       this.succeeded = false;
@@ -399,18 +451,20 @@ ParallelCall.prototype = {
  *     {Array of Exception} allErrors - The exceptions from all calls.
  */
 function PriorityOrderAbortable(successCallback, errorCallback) {
-  assert(typeof(successCallback) == "function");
-  assert(typeof(errorCallback) == "function");
+  assert(typeof successCallback == "function");
+  assert(typeof errorCallback == "function");
   ParallelAbortable.call(this); // call super constructor
+  this._successfulCall = null;
 
   this.addOneFinishedObserver(finishedCall => {
-    let haveHigherSuccess = false;
     for (let call of this._calls) {
       if (!call.finished) {
-        if (haveHigherSuccess) {
+        if (this._successfulCall) {
           // abort
           if (call.callerAbortable) {
-            call.callerAbortable.cancel(NoLongerNeededException("Another higher call succeeded"));
+            call.callerAbortable.cancel(
+              new NoLongerNeededException("Another higher call succeeded")
+            );
           }
           continue;
         }
@@ -421,14 +475,14 @@ function PriorityOrderAbortable(successCallback, errorCallback) {
         // it failed. ignore it.
         continue;
       }
-      if (haveHigherSuccess) {
-        // another successful call was higher. ignore it.
+      if (this._successfulCall) {
+        // we already have a winner. ignore it.
         continue;
       }
-      // This is the winner.
       try {
         successCallback(call.result, call);
-        haveHigherSuccess = true;
+        // This is the winner.
+        this._successfulCall = call;
       } catch (e) {
         console.error(e);
         // If the handler failed with this data, treat this call as failed.
@@ -436,7 +490,7 @@ function PriorityOrderAbortable(successCallback, errorCallback) {
         call.succeeded = false;
       }
     }
-    if (!haveHigherSuccess) {
+    if (!this._successfulCall) {
       // all failed
       errorCallback(this._calls[0].e, this._calls.map(call => call.e)); // see docs above
     }
@@ -451,8 +505,7 @@ function NoLongerNeededException(msg) {
 NoLongerNeededException.prototype = Object.create(CancelledException.prototype);
 NoLongerNeededException.prototype.constructor = NoLongerNeededException;
 
-
-/////////////////////////////////////////
+// -------------------
 // High level features
 
 /**
@@ -497,8 +550,12 @@ AddonInstaller.prototype.constructor = AddonInstaller;
  * @returns {Boolean} is OK
  */
 AddonInstaller.prototype.matches = function(addon) {
-  return !this._id || (this._id == addon.id &&
-    (!this._minVersion || Services.vc.compare(addon.version, this._minVersion) >= 0));
+  return (
+    !this._id ||
+    (this._id == addon.id &&
+      (!this._minVersion ||
+        Services.vc.compare(addon.version, this._minVersion) >= 0))
+  );
 };
 
 /**
@@ -530,22 +587,15 @@ AddonInstaller.prototype.isInstalled = async function() {
  * The downloaded XPI will be checked using prompt().
  */
 AddonInstaller.prototype._installDirect = async function() {
-  var installer = this._installer = await AddonManager.getInstallForURL(
-    this._url, null, "application/x-xpinstall", null, this._name);
+  var installer = (this._installer = await AddonManager.getInstallForURL(
+    this._url,
+    { name: this._name }
+  ));
   installer.promptHandler = makeCallback(this, this.prompt);
-  await new Promise((resolve, reject) => {
-    installer.addListener({
-      onDownloadFailed: reject,
-      onDownloadCancelled: reject,
-      onInstallFailed: reject,
-      onInstallCancelled: reject,
-      onInstallEnded: resolve,
-    });
-    installer.install();
-  });
+  await installer.install(); // throws, if failed
 
   var addon = await AddonManager.getAddonByID(this._id);
-  addon.userDisabled = false;
+  await addon.enable();
 
   // Wait for addon startup code to finish
   // Fixes: verify password fails with NOT_AVAILABLE in createIncomingServer()
@@ -563,7 +613,9 @@ AddonInstaller.prototype._installDirect = async function() {
 AddonInstaller.prototype.prompt = async function(info) {
   if (!this.matches(info.addon)) {
     // happens only when we got the wrong XPI
-    throw new Exception("The downloaded addon XPI does not match the minimum requirements");
+    throw new Exception(
+      "The downloaded addon XPI does not match the minimum requirements"
+    );
   }
 };
 
@@ -571,92 +623,110 @@ AddonInstaller.prototype.cancel = function() {
   if (this._installer) {
     try {
       this._installer.cancel();
-    } catch (e) { // if install failed
+    } catch (e) {
+      // if install failed
       ddump(e);
     }
   }
 };
 
-/////////////////////////////////////////
+// ------------
 // Debug output
 
-function deepCopy(org)
-{
-  if (typeof(org) == "undefined")
+function deepCopy(org) {
+  if (typeof org == "undefined") {
     return undefined;
-  if (org == null)
+  }
+  if (org == null) {
     return null;
-  if (typeof(org) == "string")
+  }
+  if (typeof org == "string") {
     return org;
-  if (typeof(org) == "number")
+  }
+  if (typeof org == "number") {
     return org;
-  if (typeof(org) == "boolean")
-    return org == true;
-  if (typeof(org) == "function")
+  }
+  if (typeof org == "boolean") {
     return org;
-  if (typeof(org) != "object")
-    throw "can't copy objects of type " + typeof(org) + " yet";
+  }
+  if (typeof org == "function") {
+    return org;
+  }
+  if (typeof org != "object") {
+    throw new Error("can't copy objects of type " + typeof org + " yet");
+  }
 
-  //TODO still instanceof org != instanceof copy
-  //var result = new org.constructor();
-  var result = new Object();
-  if (typeof(org.length) != "undefined")
-    var result = new Array();
-  for (var prop in org)
+  // TODO still instanceof org != instanceof copy
+  // var result = new org.constructor();
+  var result = {};
+  if (typeof org.length != "undefined") {
+    result = [];
+  }
+  for (var prop in org) {
     result[prop] = deepCopy(org[prop]);
+  }
   return result;
 }
 
 if (typeof gEmailWizardLogger == "undefined") {
-  ChromeUtils.import("resource:///modules/gloda/log4moz.js");
+  var { Log4Moz } = ChromeUtils.import("resource:///modules/gloda/log4moz.js");
   var gEmailWizardLogger = Log4Moz.getConfiguredLogger("mail.setup");
   gEmailWizardLogger.level = Log4Moz.Level.Info;
-  gEmailWizardLogger.addAppender(new Log4Moz.ConsoleAppender(new Log4Moz.BasicFormatter())); // browser console
-  //gEmailWizardLogger.addAppender(new Log4Moz.DumpAppender(new Log4Moz.BasicFormatter())); // stdout
+  gEmailWizardLogger.addAppender(
+    new Log4Moz.ConsoleAppender(new Log4Moz.BasicFormatter())
+  ); // browser console
 }
 
-function ddump(text)
-{
+function ddump(text) {
   gEmailWizardLogger.info(text);
 }
 
-function debugObject(obj, name, maxDepth, curDepth)
-{
-  if (curDepth == undefined)
+function debugObject(obj, name, maxDepth, curDepth) {
+  if (curDepth == undefined) {
     curDepth = 0;
-  if (maxDepth != undefined && curDepth > maxDepth)
+  }
+  if (maxDepth != undefined && curDepth > maxDepth) {
     return "";
+  }
 
   var result = "";
   var i = 0;
-  for (let prop in obj)
-  {
+  for (let prop in obj) {
     i++;
     try {
-      if (typeof(obj[prop]) == "object")
-      {
-        if (obj[prop] && obj[prop].length != undefined)
-          result += name + "." + prop + "=[probably array, length " +
-                obj[prop].length + "]\n";
-        else
-          result += name + "." + prop + "=[" + typeof(obj[prop]) + "]\n";
-        result += debugObject(obj[prop], name + "." + prop,
-                              maxDepth, curDepth + 1);
-      }
-      else if (typeof(obj[prop]) == "function")
+      if (typeof obj[prop] == "object") {
+        if (obj[prop] && obj[prop].length != undefined) {
+          result +=
+            name +
+            "." +
+            prop +
+            "=[probably array, length " +
+            obj[prop].length +
+            "]\n";
+        } else {
+          result += name + "." + prop + "=[" + typeof obj[prop] + "]\n";
+        }
+        result += debugObject(
+          obj[prop],
+          name + "." + prop,
+          maxDepth,
+          curDepth + 1
+        );
+      } else if (typeof obj[prop] == "function") {
         result += name + "." + prop + "=[function]\n";
-      else
+      } else {
         result += name + "." + prop + "=" + obj[prop] + "\n";
+      }
     } catch (e) {
       result += name + "." + prop + "-> Exception(" + e + ")\n";
     }
   }
-  if (!i)
+  if (!i) {
     result += name + " is empty\n";
+  }
   return result;
 }
 
-function alertPrompt(alertTitle, alertMsg)
-{
+function alertPrompt(alertTitle, alertMsg) {
   Services.prompt.alert(window, alertTitle, alertMsg);
 }
