@@ -2,8 +2,13 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this file,
  * You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-ChromeUtils.import("resource://gre/modules/Services.jsm");
-ChromeUtils.import("resource:///modules/gloda/log4moz.js");
+// mail/base/content/contentAreaClick.js
+/* globals hRefForClickEvent, openLinkExternally */
+// mail/base/content/specialTabs.js
+/* globals specialTabs */
+
+var { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
+var { Log4Moz } = ChromeUtils.import("resource:///modules/gloda/log4moz.js");
 
 /**
  * A content tab for the account provisioner.  We use Javascript-y magic to
@@ -13,14 +18,16 @@ ChromeUtils.import("resource:///modules/gloda/log4moz.js");
  * Also note that accountProvisionerTab is a singleton (hence the maxTabs: 1).
  */
 var accountProvisionerTabType = Object.create(specialTabs.contentTabType, {
-  name: {value: "accountProvisionerTab"},
-  modes: {value: {
-    accountProvisionerTab: {
-      type: "accountProvisionerTab",
-      maxTabs: 1,
-    }
-  }},
-  _log: {value: Log4Moz.getConfiguredLogger("mail.provider")},
+  name: { value: "accountProvisionerTab" },
+  modes: {
+    value: {
+      accountProvisionerTab: {
+        type: "accountProvisionerTab",
+        maxTabs: 1,
+      },
+    },
+  },
+  _log: { value: Log4Moz.getConfiguredLogger("mail.provider") },
 });
 
 /**
@@ -39,9 +46,13 @@ accountProvisionerTabType.openTab = function(aTab, aArgs) {
   this._email = aArgs.email;
   this._searchEngine = aArgs.searchEngine || "";
 
-  this._setMonitoring(aTab.browser, aArgs.realName, aArgs.email,
-                      aArgs.searchEngine);
-}
+  this._setMonitoring(
+    aTab.browser,
+    aArgs.realName,
+    aArgs.email,
+    aArgs.searchEngine
+  );
+};
 
 /**
  * We're overriding closeTab - first, we call the closeTab of contentTab,
@@ -53,11 +64,14 @@ accountProvisionerTabType.closeTab = function(aTab) {
   this._log.info("Performing account provisioner cleanup");
   this._log.info("Removing httpRequestObserver");
   Services.obs.removeObserver(this._observer, "http-on-examine-response");
-  Services.obs.removeObserver(this._observer, "http-on-examine-cached-response");
-  Services.obs.removeObserver(this.quitObserver, "mail-unloading-messenger", false);
+  Services.obs.removeObserver(
+    this._observer,
+    "http-on-examine-cached-response"
+  );
+  Services.obs.removeObserver(this.quitObserver, "mail-unloading-messenger");
   delete this._observer;
   this._log.info("Account provisioner cleanup is done.");
-}
+};
 
 /**
  * Serialize our tab into something we can restore later.
@@ -67,29 +81,34 @@ accountProvisionerTabType.persistTab = function(aTab) {
     tabURI: aTab.browser.currentURI.spec,
     realName: this._realName,
     email: this._email,
-    searchEngine: this._searchEngine
+    searchEngine: this._searchEngine,
   };
-}
+};
 
 /**
  * Re-open the accountProvisionerTab with all of the stuff we stashed in
  * persistTab. This will automatically hook up our monitoring again.
  */
 accountProvisionerTabType.restoreTab = function(aTabmail, aPersistedState) {
-  aTabmail.openTab("accountProvisionerTab",
-                   { contentPage: aPersistedState.tabURI,
-                     realName: aPersistedState.realName,
-                     email: aPersistedState.email,
-                     searchEngine: aPersistedState.searchEngine,
-                     background: true } );
-}
+  aTabmail.openTab("accountProvisionerTab", {
+    contentPage: aPersistedState.tabURI,
+    realName: aPersistedState.realName,
+    email: aPersistedState.email,
+    searchEngine: aPersistedState.searchEngine,
+    background: true,
+  });
+};
 
 /**
  * This function registers an observer to watch for HTTP requests where the
  * contentType contains text/xml.
  */
-accountProvisionerTabType._setMonitoring = function(aBrowser, aRealName,
-                                                    aEmail, aSearchEngine) {
+accountProvisionerTabType._setMonitoring = function(
+  aBrowser,
+  aRealName,
+  aEmail,
+  aSearchEngine
+) {
   let mail3Pane = Services.wm.getMostRecentWindow("mail:3pane");
 
   // We'll construct our special observer (defined in urlListener.js)
@@ -107,7 +126,7 @@ accountProvisionerTabType._setMonitoring = function(aBrowser, aRealName,
   Services.obs.addObserver(this.quitObserver, "mail-unloading-messenger");
 
   this._log.info("httpRequestObserver wired up.");
-}
+};
 
 /**
  * Click handler for the Account Provisioner tab that allows all links
@@ -117,10 +136,11 @@ accountProvisionerTabType._setMonitoring = function(aBrowser, aRealName,
 accountProvisionerTabType.clickHandler = function(aEvent) {
   // Don't handle events that: a) aren't trusted, b) have already been
   // handled or c) aren't left-click.
-  if (!aEvent.isTrusted || aEvent.defaultPrevented || aEvent.button)
+  if (!aEvent.isTrusted || aEvent.defaultPrevented || aEvent.button) {
     return true;
+  }
 
-  let href = hRefForClickEvent(aEvent, true);
+  let href = hRefForClickEvent(aEvent, true)[0];
 
   // Check to see if we're set to open the link externally...
   if (aEvent.target.hasAttribute("target")) {
@@ -131,7 +151,7 @@ accountProvisionerTabType.clickHandler = function(aEvent) {
   }
 
   return false;
-}
+};
 
 /**
  * This observer listens for the mail-unloading-messenger event fired by each
@@ -141,14 +161,16 @@ accountProvisionerTabType.clickHandler = function(aEvent) {
  * dialog again.
  */
 accountProvisionerTabType.quitObserver = {
-  observe: function(aSubject, aTopic, aData) {
+  observe(aSubject, aTopic, aData) {
     // Make sure we saw the right topic, and that the window that is closing
     // is the 3pane window that the accountProvisionerTab belongs to.
-    if (aTopic == "mail-unloading-messenger" && (aSubject === window)) {
+    if (aTopic == "mail-unloading-messenger" && aSubject === window) {
       // We quit while the accountProvisionerTab was opened. Set our sneaky
       // pref so that we suppress the dialog on startup.
-      Services.prefs.setBoolPref("mail.provider.suppress_dialog_on_startup",
-                                 true);
+      Services.prefs.setBoolPref(
+        "mail.provider.suppress_dialog_on_startup",
+        true
+      );
     }
-  }
-}
+  },
+};
