@@ -13,8 +13,8 @@
 |   longer term, this code will be restructured to make it more reusable.      |
 ------------------------------------------------------------------------------*/
 
-ChromeUtils.import("resource://gre/modules/XPCOMUtils.jsm");
-ChromeUtils.import("resource://gre/modules/Services.jsm");
+var {XPCOMUtils} = ChromeUtils.import("resource://gre/modules/XPCOMUtils.jsm");
+var {Services} = ChromeUtils.import("resource://gre/modules/Services.jsm");
 
 var gContextMenuContentData = null;
 
@@ -634,7 +634,7 @@ nsContextMenu.prototype = {
         if (this.onTextInput && !this.target.readOnly &&
             this.target.mozIsTextField(true) && this.target.spellcheck) {
           this.possibleSpellChecking = true;
-          InlineSpellCheckerUI.init(this.target.QueryInterface(Ci.nsIDOMNSEditableElement).editor);
+          InlineSpellCheckerUI.init(this.target.editor);
           InlineSpellCheckerUI.initFromEvent(aRangeParent, aRangeOffset);
         }
         this.onKeywordField = this.isTargetAKeywordField(this.target);
@@ -643,7 +643,7 @@ nsContextMenu.prototype = {
         this.onTextInput = this.isTextBoxEnabled(this.target);
         if (this.onTextInput && !this.target.readOnly && this.target.spellcheck) {
           this.possibleSpellChecking = true;
-          InlineSpellCheckerUI.init(this.target.QueryInterface(Ci.nsIDOMNSEditableElement).editor);
+          InlineSpellCheckerUI.init(this.target.editor);
           InlineSpellCheckerUI.initFromEvent(aRangeParent, aRangeOffset);
         }
       }
@@ -924,34 +924,28 @@ nsContextMenu.prototype = {
 
   // Reload image
   reloadImage: function() {
-    urlSecurityCheck(this.mediaURL,
-                     this.target.nodePrincipal,
-                     Ci.nsIScriptSecurityManager.DISALLOW_SCRIPT);
+    urlSecurityCheck(this.mediaURL, this.target.nodePrincipal,
+                     Ci.nsIScriptSecurityManager.ALLOW_CHROME);
     if (this.target instanceof Ci.nsIImageLoadingContent)
       this.target.forceReload();
   },
 
   // Change current window to the URL of the image, video, or audio.
-  viewMedia(e) {
-    let doc = this.target.ownerDocument;
-    let where = whereToOpenLink(e);
-    if (this.onCanvas) {
-      let blobUrl = URL.createObjectURL(this.target);
-      let systemPrincipal = Services.scriptSecurityManager
-                                    .getSystemPrincipal();
-      openUILinkIn(blobUrl, where,
-                   { referrerURI: doc.documentURIObject,
-                     triggeringPrincipal: systemPrincipal,
-                   });
-    } else {
-      urlSecurityCheck(this.mediaURL,
-                       this.target.nodePrincipal,
-                       Ci.nsIScriptSecurityManager.DISALLOW_SCRIPT);
-      openUILinkIn(this.mediaURL, where,
-                   { referrerURI: doc.documentURIObject,
-                     triggeringPrincipal: this.target.nodePrincipal,
-                   });
+  viewMedia: function(aEvent) {
+    var viewURL;
+    if (this.onCanvas)
+      viewURL = this.target.toDataURL();
+    else {
+      viewURL = this.mediaURL;
+      urlSecurityCheck(viewURL, this.target.nodePrincipal,
+                       Ci.nsIScriptSecurityManager.ALLOW_CHROME);
     }
+    var doc = this.target.ownerDocument;
+    var where = whereToOpenLink(aEvent);
+    if (where == "current")
+      openTopWin(viewURL, doc.defaultView);
+    else
+      openUILinkIn(viewURL, where, null, null, doc.documentURIObject);
   },
 
   saveVideoFrameAsImage: function () {
@@ -973,7 +967,8 @@ nsContextMenu.prototype = {
     saveImageURL(canvas.toDataURL("image/jpeg", ""), name, "SaveImageTitle",
                                   true, true,
                                   this.target.ownerDocument.documentURIObject,
-                                  null, null, null, (gPrivate ? true : false));
+                                  null, null, null, (gPrivate ? true : false),
+                                  this.target.ownerDocument.nodePrincipal);
   },
 
   // Full screen video playback
@@ -986,17 +981,15 @@ nsContextMenu.prototype = {
   },
 
   // Change current window to the URL of the background image.
-  viewBGImage(e) {
-    urlSecurityCheck(this.bgImageURL,
-                     this.target.nodePrincipal,
-                     Ci.nsIScriptSecurityManager.DISALLOW_SCRIPT);
-
-    let doc = this.target.ownerDocument;
-    let where = whereToOpenLink(e);
-    openUILinkIn(this.bgImageURL, where,
-                 { referrerURI: doc.documentURIObject,
-                   triggeringPrincipal: this.target.nodePrincipal,
-                 });
+  viewBGImage: function(aEvent) {
+    urlSecurityCheck(this.bgImageURL, this.target.nodePrincipal,
+                     Ci.nsIScriptSecurityManager.ALLOW_CHROME);
+    var doc = this.target.ownerDocument;
+    var where = whereToOpenLink(aEvent);
+    if (where == "current")
+      openTopWin(this.bgImageURL, doc.defaultView);
+    else
+      openUILinkIn(this.bgImageURL, where, null, null, doc.documentURIObject);
   },
 
   setDesktopBackground: function() {
@@ -1151,12 +1144,14 @@ nsContextMenu.prototype = {
       // Bypass cache, since it's a data: URL.
       saveImageURL(this.target.toDataURL(), "canvas.png", "SaveImageTitle",
                    true, false, referrerURI, null, null, null,
-                   (gPrivate ? true : false));
+                   (gPrivate ? true : false),
+                   doc.nodePrincipal /* system, because blob: */);
     else if (this.onImage) {
       saveImageURL(this.mediaURL, null, "SaveImageTitle", false,
                    false, referrerURI, null, gContextMenuContentData.contentType,
                    gContextMenuContentData.contentDisposition,
-                   (gPrivate ? true : false));
+                   (gPrivate ? true : false),
+                   doc.nodePrincipal);
     }
     else if (this.onVideo || this.onAudio) {
       var dialogTitle = this.onVideo ? "SaveVideoTitle" : "SaveAudioTitle";
