@@ -3,8 +3,8 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-ChromeUtils.import("resource://gre/modules/Services.jsm");
-ChromeUtils.import("resource://gre/modules/XPCOMUtils.jsm");
+var {Services} = ChromeUtils.import("resource://gre/modules/Services.jsm");
+var {XPCOMUtils} = ChromeUtils.import("resource://gre/modules/XPCOMUtils.jsm");
 
 const WCCR_CONTRACTID = "@mozilla.org/embeddor.implemented/web-content-handler-registrar;1";
 const WCCR_CLASSID = Components.ID("{792a7e82-06a0-437c-af63-b2d12e808acc}");
@@ -62,8 +62,7 @@ WebContentConverter.prototype = {
 
   QueryInterface: XPCOMUtils.generateQI(
     [Ci.nsIStreamConverter,
-     Ci.nsIStreamListener,
-     Ci.nsISupports])
+     Ci.nsIStreamListener])
 };
 
 var WebContentConverterFactory = {
@@ -74,8 +73,7 @@ var WebContentConverterFactory = {
   },
 
   QueryInterface: XPCOMUtils.generateQI(
-    [Ci.nsIFactory,
-     Ci.nsISupports])
+    [Ci.nsIFactory])
 };
 
 function ServiceInfo(contentType, uri, name) {
@@ -129,8 +127,7 @@ ServiceInfo.prototype = {
   },
 
   QueryInterface: XPCOMUtils.generateQI(
-    [Ci.nsIWebContentHandlerInfo,
-     Ci.nsISupports])
+    [Ci.nsIWebContentHandlerInfo])
 };
 
 function WebContentConverterRegistrar() {
