@@ -4,8 +4,8 @@
 
 const XULNS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";
 
-ChromeUtils.import("resource://gre/modules/XPCOMUtils.jsm");
-ChromeUtils.import("resource://gre/modules/Services.jsm");
+var {Services} = ChromeUtils.import("resource://gre/modules/Services.jsm");
+var {XPCOMUtils} = ChromeUtils.import("resource://gre/modules/XPCOMUtils.jsm");
 ChromeUtils.import("resource:///modules/mailnewsMigrator.js");
 ChromeUtils.import("resource:///modules/extensionSupport.jsm");
 
@@ -37,9 +37,9 @@ XPCOMUtils.defineLazyGetter(this, "DebuggerServer", () => {
   return tmp.require("devtools/server/main").DebuggerServer;
 });
 
-var global = this;
+const global = this;
 
-var listeners = {
+const listeners = {
   mm: {
     // PLEASE KEEP THIS LIST IN SYNC WITH THE MOBILE LISTENERS IN nsBrowserGlue.js
     "RemoteLogins:findLogins": ["LoginManagerParent"],
@@ -150,7 +150,7 @@ SuiteGlue.prototype = {
     }
     delay = delay <= MAX_DELAY ? delay : MAX_DELAY;
 
-    ChromeUtils.import("resource://services-sync/main.js");
+    const {Weave} = ChromeUtils.import("resource://services-sync/main.js");
     Weave.Service.scheduler.delayedAutoConnect(delay);
   },
 
@@ -443,7 +443,8 @@ SuiteGlue.prototype = {
 
         // Sadly we still need to clear the database manually. Experiments
         // showed that the permissions manager deletes only one record.
-        db.beginTransactionAs(Ci.mozIStorageConnection.TRANSACTION_EXCLUSIVE);
+        db.defaultTransactionType = Ci.mozIStorageConnection.TRANSACTION_EXCLUSIVE;
+        db.beginTransaction();
 
         try {
           db.executeSimpleSQL("delete from moz_perms where " +
@@ -555,7 +556,7 @@ SuiteGlue.prototype = {
     let enumerator = aSource.directoryEntries;
 
     while (enumerator.hasMoreElements()) {
-      let file = enumerator.getNext().QueryInterface(Ci.nsIFile);
+      let file = enumerator.nextFile;
 
       if (file.isDirectory()) {
         let subdir = aDestination.clone();
@@ -1166,7 +1167,7 @@ SuiteGlue.prototype = {
     var dictList = o1.value;
     // If the preference contains an invalid dictionary, set it to a valid
     // dictionary, any dictionary will do.
-    if (dictList.length && !dictList.includes(prefValue))
+    if (dictList.length && dictList.indexOf(prefValue) < 0)
       Services.prefs.setCharPref(prefName, dictList[0]);
   },
 
@@ -1475,7 +1476,7 @@ SuiteGlue.prototype = {
  * PermissionUI.jsm for an example of how to provide a new prompt
  * from an add-on.
  */
-var ContentPermissionIntegration = {
+const ContentPermissionIntegration = {
   /**
    * Creates a PermissionPrompt for a given permission type and
    * nsIContentPermissionRequest.
