@@ -23,7 +23,7 @@
 // Import modules
 //////////////////////////////////////////////////////////////
 
-ChromeUtils.import("resource://gre/modules/Services.jsm");
+var {Services} = ChromeUtils.import("resource://gre/modules/Services.jsm");
 
 //////////////////////////////////////////////////////////////
 // Global variables
@@ -531,7 +531,7 @@ function sb_panel_is_excluded(node)
 {
   var exclude = node.getAttribute('exclude');
   return ( exclude && exclude != '' &&
-           exclude.includes(sidebarObj.component));
+           exclude.indexOf(sidebarObj.component) != -1 );
 }
 sbPanel.prototype.is_excluded =
 function ()
