@@ -4,38 +4,43 @@
 
 "use strict";
 
-const {l10n} = require("devtools/shared/inspector/css-logic");
-const {ELEMENT_STYLE} = require("devtools/shared/specs/styles");
+const { l10n } = require("devtools/shared/inspector/css-logic");
+const { ELEMENT_STYLE } = require("devtools/shared/specs/styles");
 const Rule = require("devtools/client/inspector/rules/models/rule");
 const {
   InplaceEditor,
   editableField,
-  editableItem
+  editableItem,
 } = require("devtools/client/shared/inplace-editor");
-const TextPropertyEditor =
-  require("devtools/client/inspector/rules/views/text-property-editor");
+const TextPropertyEditor = require("devtools/client/inspector/rules/views/text-property-editor");
 const {
   createChild,
   blurOnMultipleProperties,
-  promiseWarn
+  promiseWarn,
 } = require("devtools/client/inspector/shared/utils");
 const {
   parseNamedDeclarations,
   parsePseudoClassesAndAttributes,
   SELECTOR_ATTRIBUTE,
   SELECTOR_ELEMENT,
-  SELECTOR_PSEUDO_CLASS
+  SELECTOR_PSEUDO_CLASS,
 } = require("devtools/shared/css/parsing-utils");
 const promise = require("promise");
 const Services = require("Services");
-const EventEmitter = require("devtools/shared/old-event-emitter");
-const {Task} = require("devtools/shared/task");
-const {Tools} = require("devtools/client/definitions");
-const {gDevTools} = require("devtools/client/framework/devtools");
+const EventEmitter = require("devtools/shared/event-emitter");
 const CssLogic = require("devtools/shared/inspector/css-logic");
 
-const STYLE_INSPECTOR_PROPERTIES = "devtools/shared/locales/styleinspector.properties";
-const {LocalizationHelper} = require("devtools/shared/l10n");
+loader.lazyRequireGetter(this, "Tools", "devtools/client/definitions", true);
+loader.lazyRequireGetter(
+  this,
+  "gDevTools",
+  "devtools/client/framework/devtools",
+  true
+);
+
+const STYLE_INSPECTOR_PROPERTIES =
+  "devtools/shared/locales/styleinspector.properties";
+const { LocalizationHelper } = require("devtools/shared/l10n");
 const STYLE_INSPECTOR_L10N = new LocalizationHelper(STYLE_INSPECTOR_PROPERTIES);
 
 /**
@@ -55,9 +60,7 @@ function RuleEditor(ruleView, rule) {
   this.ruleView = ruleView;
   this.doc = this.ruleView.styleDocument;
   this.toolbox = this.ruleView.inspector.toolbox;
-  // We stash this locally so that we can refer to it from |destroy|
-  // without accidentally reinstantiating the service during shutdown.
-  this.sourceMapURLService = this.toolbox.sourceMapURLService;
+  this.telemetry = this.toolbox.telemetry;
   this.rule = rule;
 
   this.isEditable = !rule.isSystem;
@@ -82,7 +85,7 @@ function RuleEditor(ruleView, rule) {
 }
 
 RuleEditor.prototype = {
-  destroy: function () {
+  destroy: function() {
     this.rule.domRule.off("location-changed");
     this.toolbox.off("tool-registered", this._onToolChanged);
     this.toolbox.off("tool-unregistered", this._onToolChanged);
@@ -91,19 +94,39 @@ RuleEditor.prototype = {
     if (this.rule.sheet) {
       url = this.rule.sheet.href || this.rule.sheet.nodeHref;
     }
-    if (url && !this.rule.isSystem && this.rule.domRule.type !== ELEMENT_STYLE) {
+    if (
+      url &&
+      !this.rule.isSystem &&
+      this.rule.domRule.type !== ELEMENT_STYLE
+    ) {
       // Only get the original source link if the rule isn't a system
       // rule and if it isn't an inline rule.
-      let sourceLine = this.rule.ruleLine;
-      let sourceColumn = this.rule.ruleColumn;
-      this.sourceMapURLService.unsubscribe(url, sourceLine, sourceColumn,
-                                           this._updateLocation);
+      const sourceLine = this.rule.ruleLine;
+      const sourceColumn = this.rule.ruleColumn;
+
+      if (this._sourceMapURLService) {
+        this._sourceMapURLService.unsubscribe(
+          url,
+          sourceLine,
+          sourceColumn,
+          this._updateLocation
+        );
+      }
     }
   },
 
+  get sourceMapURLService() {
+    if (!this._sourceMapURLService) {
+      // sourceMapURLService is a lazy getter in the toolbox.
+      this._sourceMapURLService = this.toolbox.sourceMapURLService;
+    }
+
+    return this._sourceMapURLService;
+  },
+
   get isSelectorEditable() {
-    let trait = this.isEditable &&
-      this.ruleView.inspector.target.client.traits.selectorEditable &&
+    const trait =
+      this.isEditable &&
       this.rule.domRule.type !== ELEMENT_STYLE &&
       this.rule.domRule.type !== CSSRule.KEYFRAME_RULE;
 
@@ -112,7 +135,7 @@ RuleEditor.prototype = {
     return trait && !this.rule.elementStyle.element.isAnonymous;
   },
 
-  _create: function () {
+  _create: function() {
     this.element = this.doc.createElement("div");
     this.element.className = "ruleview-rule devtools-monospace";
     this.element.setAttribute("uneditable", !this.isEditable);
@@ -125,21 +148,21 @@ RuleEditor.prototype = {
 
     // Add the source link.
     this.source = createChild(this.element, "div", {
-      class: "ruleview-rule-source theme-link"
+      class: "ruleview-rule-source theme-link",
     });
     this.source.addEventListener("click", this._onSourceClick);
 
-    let sourceLabel = this.doc.createElement("span");
+    const sourceLabel = this.doc.createElement("span");
     sourceLabel.classList.add("ruleview-rule-source-label");
     this.source.appendChild(sourceLabel);
 
     this.updateSourceLink();
 
-    let code = createChild(this.element, "div", {
-      class: "ruleview-code"
+    const code = createChild(this.element, "div", {
+      class: "ruleview-code",
     });
 
-    let header = createChild(code, "div", {});
+    const header = createChild(code, "div", {});
 
     this.selectorText = createChild(header, "span", {
       class: "ruleview-selectorcontainer",
@@ -160,7 +183,7 @@ RuleEditor.prototype = {
     }
 
     if (this.rule.domRule.type !== CSSRule.KEYFRAME_RULE) {
-      Task.spawn(function* () {
+      (async function() {
         let selector;
 
         if (this.rule.domRule.selectors) {
@@ -169,34 +192,46 @@ RuleEditor.prototype = {
         } else if (this.rule.inherited) {
           // This is an inline style from an inherited rule. Need to resolve the unique
           // selector from the node which rule this is inherited from.
-          selector = yield this.rule.inherited.getUniqueSelector();
+          selector = await this.rule.inherited.getUniqueSelector();
         } else {
           // This is an inline style from the current node.
           selector = this.ruleView.inspector.selectionCssSelector;
         }
 
-        let selectorHighlighter = createChild(header, "span", {
-          class: "ruleview-selectorhighlighter" +
-                 (this.ruleView.highlighters.selectorHighlighterShown === selector ?
-                  " highlighted" : ""),
-          title: l10n("rule.selectorHighlighter.tooltip")
+        const isHighlighted =
+          this.ruleView._highlighters &&
+          this.ruleView.highlighters.selectorHighlighterShown === selector;
+        const selectorHighlighter = createChild(header, "span", {
+          class:
+            "ruleview-selectorhighlighter" +
+            (isHighlighted ? " highlighted" : ""),
+          title: l10n("rule.selectorHighlighter.tooltip"),
         });
-        selectorHighlighter.addEventListener("click", () => {
-          this.ruleView.toggleSelectorHighlighter(selectorHighlighter, selector);
+        selectorHighlighter.addEventListener("click", event => {
+          this.ruleView.toggleSelectorHighlighter(
+            selectorHighlighter,
+            selector
+          );
+          // Prevent clicks from focusing the property editor.
+          event.stopPropagation();
         });
 
         this.uniqueSelector = selector;
         this.emit("selector-icon-created");
-      }.bind(this));
+      }
+        .bind(this)()
+        .catch(error => {
+          console.error("Exception while getting unique selector", error);
+        }));
     }
 
     this.openBrace = createChild(header, "span", {
       class: "ruleview-ruleopen",
-      textContent: " {"
+      textContent: " {",
     });
 
     this.propertyList = createChild(code, "ul", {
-      class: "ruleview-propertylist"
+      class: "ruleview-propertylist",
     });
 
     this.populate();
@@ -204,7 +239,7 @@ RuleEditor.prototype = {
     this.closeBrace = createChild(code, "div", {
       class: "ruleview-ruleclose",
       tabindex: this.isEditable ? "0" : "-1",
-      textContent: "}"
+      textContent: "}",
     });
 
     if (this.isEditable) {
@@ -218,7 +253,7 @@ RuleEditor.prototype = {
       });
 
       code.addEventListener("click", () => {
-        let selection = this.doc.defaultView.getSelection();
+        const selection = this.doc.defaultView.getSelection();
         if (selection.isCollapsed && !this._ruleViewIsEditing) {
           this.newProperty();
         }
@@ -240,7 +275,7 @@ RuleEditor.prototype = {
   /**
    * Called when a tool is registered or unregistered.
    */
-  _onToolChanged: function () {
+  _onToolChanged: function() {
     // When the source editor is registered, update the source links
     // to be clickable; and if it is unregistered, update the links to
     // be unclickable.  However, some links are never clickable, so
@@ -258,20 +293,27 @@ RuleEditor.prototype = {
    * Event handler called when a property changes on the
    * StyleRuleActor.
    */
-  _locationChanged: function () {
+  _locationChanged: function() {
     this.updateSourceLink();
   },
 
-  _onSourceClick: function () {
+  _onSourceClick: function() {
     if (this.source.hasAttribute("unselectable") || !this._currentLocation) {
       return;
     }
 
-    let target = this.ruleView.inspector.target;
+    const target = this.ruleView.inspector.target;
     if (Tools.styleEditor.isTargetSupported(target)) {
       gDevTools.showToolbox(target, "styleeditor").then(toolbox => {
-        let {url, line, column} = this._currentLocation;
-        toolbox.getCurrentPanel().selectStyleSheet(url, line, column);
+        const { url, line, column } = this._currentLocation;
+
+        if (!this.rule.sheet.href && this.rule.sheet.nodeHref) {
+          toolbox
+            .getCurrentPanel()
+            .selectStyleSheet(this.rule.sheet, line, column);
+        } else {
+          toolbox.getCurrentPanel().selectStyleSheet(url, line, column);
+        }
       });
     }
   },
@@ -293,7 +335,7 @@ RuleEditor.prototype = {
    * @param {number} column
    *        The original column number
    */
-  _updateLocation: function (enabled, url, line, column) {
+  _updateLocation: function(enabled, url, line, column) {
     let displayURL = url;
     if (!enabled) {
       url = null;
@@ -309,10 +351,10 @@ RuleEditor.prototype = {
     this._currentLocation = {
       url,
       line,
-      column
+      column,
     };
 
-    let sourceTextContent = CssLogic.shortSource({href: displayURL});
+    let sourceTextContent = CssLogic.shortSource({ href: displayURL });
     let title = displayURL ? displayURL : sourceTextContent;
     if (line > 0) {
       sourceTextContent += ":" + line;
@@ -323,19 +365,23 @@ RuleEditor.prototype = {
       title += " @" + this.rule.mediaText;
     }
 
-    let sourceLabel = this.element.querySelector(".ruleview-rule-source-label");
+    const sourceLabel = this.element.querySelector(
+      ".ruleview-rule-source-label"
+    );
     sourceLabel.setAttribute("title", title);
     sourceLabel.textContent = sourceTextContent;
   },
 
-  updateSourceLink: function () {
+  updateSourceLink: function() {
     if (this.rule.isSystem) {
-      let sourceLabel = this.element.querySelector(".ruleview-rule-source-label");
-      let title = this.rule.title;
-      let sourceHref = (this.rule.sheet && this.rule.sheet.href) ?
-          this.rule.sheet.href : title;
+      const sourceLabel = this.element.querySelector(
+        ".ruleview-rule-source-label"
+      );
+      const title = this.rule.title;
+      const sourceHref =
+        this.rule.sheet && this.rule.sheet.href ? this.rule.sheet.href : title;
 
-      let uaLabel = STYLE_INSPECTOR_L10N.getStr("rule.userAgentStyles");
+      const uaLabel = STYLE_INSPECTOR_L10N.getStr("rule.userAgentStyles");
       sourceLabel.textContent = uaLabel + " " + title;
 
       // Special case about:PreferenceStyleSheet, as it is generated on the
@@ -354,13 +400,21 @@ RuleEditor.prototype = {
     if (this.rule.sheet) {
       url = this.rule.sheet.href || this.rule.sheet.nodeHref;
     }
-    if (url && !this.rule.isSystem && this.rule.domRule.type !== ELEMENT_STYLE) {
+    if (
+      url &&
+      !this.rule.isSystem &&
+      this.rule.domRule.type !== ELEMENT_STYLE
+    ) {
       // Only get the original source link if the rule isn't a system
       // rule and if it isn't an inline rule.
-      let sourceLine = this.rule.ruleLine;
-      let sourceColumn = this.rule.ruleColumn;
-      this.sourceMapURLService.subscribe(url, sourceLine, sourceColumn,
-                                         this._updateLocation);
+      const sourceLine = this.rule.ruleLine;
+      const sourceColumn = this.rule.ruleColumn;
+      this.sourceMapURLService.subscribe(
+        url,
+        sourceLine,
+        sourceColumn,
+        this._updateLocation
+      );
       // Set "unselectable" appropriately.
       this._onToolChanged();
     } else if (this.rule.domRule.type === ELEMENT_STYLE) {
@@ -377,8 +431,11 @@ RuleEditor.prototype = {
 
   /**
    * Update the rule editor with the contents of the rule.
+   *
+   * @param {Boolean} reset
+   *        True to completely reset the rule editor before populating.
    */
-  populate: function () {
+  populate: function(reset) {
     // Clear out existing viewers.
     while (this.selectorText.hasChildNodes()) {
       this.selectorText.removeChild(this.selectorText.lastChild);
@@ -396,20 +453,21 @@ RuleEditor.prototype = {
         if (i !== 0) {
           createChild(this.selectorText, "span", {
             class: "ruleview-selector-separator",
-            textContent: ", "
+            textContent: ", ",
           });
         }
 
-        let containerClass =
-          (this.rule.matchedSelectors.indexOf(selector) > -1) ?
-          "ruleview-selector-matched" : "ruleview-selector-unmatched";
-        let selectorContainer = createChild(this.selectorText, "span", {
-          class: containerClass
+        const containerClass =
+          this.rule.matchedSelectors.indexOf(selector) > -1
+            ? "ruleview-selector-matched"
+            : "ruleview-selector-unmatched";
+        const selectorContainer = createChild(this.selectorText, "span", {
+          class: containerClass,
         });
 
-        let parsedSelector = parsePseudoClassesAndAttributes(selector);
+        const parsedSelector = parsePseudoClassesAndAttributes(selector);
 
-        for (let selectorText of parsedSelector) {
+        for (const selectorText of parsedSelector) {
           let selectorClass = "";
 
           switch (selectorText.type) {
@@ -420,10 +478,14 @@ RuleEditor.prototype = {
               selectorClass = "ruleview-selector";
               break;
             case SELECTOR_PSEUDO_CLASS:
-              selectorClass = [":active", ":focus", ":hover"].some(
-                  pseudo => selectorText.value === pseudo) ?
-                "ruleview-selector-pseudo-class-lock" :
-                "ruleview-selector-pseudo-class";
+              selectorClass = [
+                ":active",
+                ":focus",
+                ":focus-within",
+                ":hover",
+              ].some(pseudo => selectorText.value === pseudo)
+                ? "ruleview-selector-pseudo-class-lock"
+                : "ruleview-selector-pseudo-class";
               break;
             default:
               break;
@@ -431,16 +493,26 @@ RuleEditor.prototype = {
 
           createChild(selectorContainer, "span", {
             textContent: selectorText.value,
-            class: selectorClass
+            class: selectorClass,
           });
         }
       });
     }
 
-    for (let prop of this.rule.textProps) {
+    if (reset) {
+      while (this.propertyList.hasChildNodes()) {
+        this.propertyList.removeChild(this.propertyList.lastChild);
+      }
+    }
+
+    for (const prop of this.rule.textProps) {
       if (!prop.editor && !prop.invisible) {
-        let editor = new TextPropertyEditor(this, prop);
+        const editor = new TextPropertyEditor(this, prop);
         this.propertyList.appendChild(editor.element);
+      } else if (prop.editor) {
+        // If an editor already existed, append it to the bottom now to make sure the
+        // order of editors in the DOM follow the order of the rule's properties.
+        this.propertyList.appendChild(prop.editor.element);
       }
     }
   },
@@ -461,19 +533,26 @@ RuleEditor.prototype = {
    * @return {TextProperty}
    *        The new property
    */
-  addProperty: function (name, value, priority, enabled, siblingProp) {
-    let prop = this.rule.createProperty(name, value, priority, enabled,
-      siblingProp);
-    let index = this.rule.textProps.indexOf(prop);
-    let editor = new TextPropertyEditor(this, prop);
+  addProperty: function(name, value, priority, enabled, siblingProp) {
+    const prop = this.rule.createProperty(
+      name,
+      value,
+      priority,
+      enabled,
+      siblingProp
+    );
+    const index = this.rule.textProps.indexOf(prop);
+    const editor = new TextPropertyEditor(this, prop);
 
     // Insert this node before the DOM node that is currently at its new index
     // in the property list.  There is currently one less node in the DOM than
     // in the property list, so this causes it to appear after siblingProp.
     // If there is no node at its index, as is the case where this is the last
     // node being inserted, then this behaves as appendChild.
-    this.propertyList.insertBefore(editor.element,
-      this.propertyList.children[index]);
+    this.propertyList.insertBefore(
+      editor.element,
+      this.propertyList.children[index]
+    );
 
     return prop;
   },
@@ -493,17 +572,22 @@ RuleEditor.prototype = {
    * @param {TextProperty} siblingProp
    *        Optional, the property next to which all new props should be added.
    */
-  addProperties: function (properties, siblingProp) {
+  addProperties: function(properties, siblingProp) {
     if (!properties || !properties.length) {
       return;
     }
 
     let lastProp = siblingProp;
-    for (let p of properties) {
-      let isCommented = Boolean(p.commentOffsets);
-      let enabled = !isCommented;
-      lastProp = this.addProperty(p.name, p.value, p.priority, enabled,
-        lastProp);
+    for (const p of properties) {
+      const isCommented = Boolean(p.commentOffsets);
+      const enabled = !isCommented;
+      lastProp = this.addProperty(
+        p.name,
+        p.value,
+        p.priority,
+        enabled,
+        lastProp
+      );
     }
 
     // Either focus on the last value if incomplete, or start a new one.
@@ -519,7 +603,7 @@ RuleEditor.prototype = {
    * name is given, we'll create a real TextProperty and add it to the
    * rule.
    */
-  newProperty: function () {
+  newProperty: function() {
     // If we're already creating a new property, ignore this.
     if (!this.closeBrace.hasAttribute("tabindex")) {
       return;
@@ -536,7 +620,7 @@ RuleEditor.prototype = {
 
     this.newPropSpan = createChild(this.newPropItem, "span", {
       class: "ruleview-propertyname",
-      tabindex: "0"
+      tabindex: "0",
     });
 
     this.multipleAddedProperties = null;
@@ -552,8 +636,10 @@ RuleEditor.prototype = {
     });
 
     // Auto-close the input if multiple rules get pasted into new property.
-    this.editor.input.addEventListener("paste",
-      blurOnMultipleProperties(this.rule.cssProperties));
+    this.editor.input.addEventListener(
+      "paste",
+      blurOnMultipleProperties(this.rule.cssProperties)
+    );
   },
 
   /**
@@ -564,7 +650,7 @@ RuleEditor.prototype = {
    * @param {Boolean} commit
    *        True if the value should be committed.
    */
-  _onNewProperty: function (value, commit) {
+  _onNewProperty: function(value, commit) {
     if (!value || !commit) {
       return;
     }
@@ -572,12 +658,19 @@ RuleEditor.prototype = {
     // parseDeclarations allows for name-less declarations, but in the present
     // case, we're creating a new declaration, it doesn't make sense to accept
     // these entries
-    this.multipleAddedProperties =
-      parseNamedDeclarations(this.rule.cssProperties.isKnown, value, true);
+    this.multipleAddedProperties = parseNamedDeclarations(
+      this.rule.cssProperties.isKnown,
+      value,
+      true
+    );
 
     // Blur the editor field now and deal with adding declarations later when
     // the field gets destroyed (see _newPropertyDestroy)
     this.editor.input.blur();
+
+    this.telemetry.recordEvent("edit_rule", "ruleview", null, {
+      session_id: this.toolbox.sessionId,
+    });
   },
 
   /**
@@ -586,7 +679,7 @@ RuleEditor.prototype = {
    * added, since we want to wait until after the inplace editor `destroy`
    * event has been fired to keep consistent UI state.
    */
-  _newPropertyDestroy: function () {
+  _newPropertyDestroy: function() {
     // We're done, make the close brace focusable again.
     this.closeBrace.setAttribute("tabindex", "0");
 
@@ -614,43 +707,36 @@ RuleEditor.prototype = {
    * @param {Number} direction
    *        The move focus direction number.
    */
-  _onSelectorDone: Task.async(function* (value, commit, direction) {
-    if (!commit || this.isEditing || value === "" ||
-        value === this.rule.selectorText) {
+  async _onSelectorDone(value, commit, direction) {
+    if (
+      !commit ||
+      this.isEditing ||
+      value === "" ||
+      value === this.rule.selectorText
+    ) {
       return;
     }
 
-    let ruleView = this.ruleView;
-    let elementStyle = ruleView._elementStyle;
-    let element = elementStyle.element;
-    let supportsUnmatchedRules =
-      this.rule.domRule.supportsModifySelectorUnmatched;
+    const ruleView = this.ruleView;
+    const elementStyle = ruleView._elementStyle;
+    const element = elementStyle.element;
 
     this.isEditing = true;
 
     try {
-      let response = yield this.rule.domRule.modifySelector(element, value);
-
-      if (!supportsUnmatchedRules) {
-        this.isEditing = false;
-
-        if (response) {
-          this.ruleView.refreshPanel();
-        }
-        return;
-      }
+      const response = await this.rule.domRule.modifySelector(element, value);
 
       // We recompute the list of applied styles, because editing a
       // selector might cause this rule's position to change.
-      let applied = yield elementStyle.pageStyle.getApplied(element, {
+      const applied = await elementStyle.pageStyle.getApplied(element, {
         inherited: true,
         matchedSelectors: true,
-        filter: elementStyle.showUserAgentStyles ? "ua" : undefined
+        filter: elementStyle.showUserAgentStyles ? "ua" : undefined,
       });
 
       this.isEditing = false;
 
-      let {ruleProps, isMatching} = response;
+      const { ruleProps, isMatching } = response;
       if (!ruleProps) {
         // Notify for changes, even when nothing changes,
         // just to allow tests being able to track end of this request.
@@ -659,12 +745,12 @@ RuleEditor.prototype = {
       }
 
       ruleProps.isUnmatched = !isMatching;
-      let newRule = new Rule(elementStyle, ruleProps);
-      let editor = new RuleEditor(ruleView, newRule);
-      let rules = elementStyle.rules;
+      const newRule = new Rule(elementStyle, ruleProps);
+      const editor = new RuleEditor(ruleView, newRule);
+      const rules = elementStyle.rules;
 
-      let newRuleIndex = applied.findIndex((r) => r.rule == ruleProps.rule);
-      let oldIndex = rules.indexOf(this.rule);
+      let newRuleIndex = applied.findIndex(r => r.rule == ruleProps.rule);
+      const oldIndex = rules.indexOf(this.rule);
 
       // If the selector no longer matches, then we leave the rule in
       // the same relative position.
@@ -676,7 +762,7 @@ RuleEditor.prototype = {
       rules.splice(oldIndex, 1);
       rules.splice(newRuleIndex, 0, newRule);
       elementStyle._changed();
-      elementStyle.markOverriddenAll();
+      elementStyle.onRuleUpdated();
 
       // We install the new editor in place of the old -- you might
       // think we would replicate the list-modification logic above,
@@ -686,8 +772,10 @@ RuleEditor.prototype = {
 
       // Remove highlight for modified selector
       if (ruleView.highlighters.selectorHighlighterShown) {
-        ruleView.toggleSelectorHighlighter(ruleView.lastSelectorIcon,
-          ruleView.highlighters.selectorHighlighterShown);
+        ruleView.toggleSelectorHighlighter(
+          ruleView.lastSelectorIcon,
+          ruleView.highlighters.selectorHighlighterShown
+        );
       }
 
       editor._moveSelectorFocus(direction);
@@ -695,7 +783,7 @@ RuleEditor.prototype = {
       this.isEditing = false;
       promiseWarn(err);
     }
-  }),
+  },
 
   /**
    * Handle moving the focus change after a tab or return keypress in the
@@ -704,7 +792,7 @@ RuleEditor.prototype = {
    * @param {Number} direction
    *        The move focus direction number.
    */
-  _moveSelectorFocus: function (direction) {
+  _moveSelectorFocus: function(direction) {
     if (!direction || direction === Services.focus.MOVEFOCUS_BACKWARD) {
       return;
     }
@@ -714,7 +802,7 @@ RuleEditor.prototype = {
     } else {
       this.propertyList.click();
     }
-  }
+  },
 };
 
 module.exports = RuleEditor;
