@@ -17,21 +17,39 @@ const ruleTester = new RuleTester({ parserOptions: { ecmaVersion: 6 } });
 // ------------------------------------------------------------------------------
 
 function invalidCode(code, originalName, newName) {
-  return {code, errors: [{
-    message: `Use ${newName} rather than ${originalName}`,
-    type: "MemberExpression"
-  }]};
+  return {
+    code,
+    errors: [
+      {
+        message: `Use ${newName} rather than ${originalName}`,
+        type: "MemberExpression",
+      },
+    ],
+  };
 }
 
 ruleTester.run("use-cc-etc", rule, {
-  valid: [
-    "Components.Constructor();",
-    "let x = Components.foo;"
-  ],
+  valid: ["Components.Constructor();", "let x = Components.foo;"],
   invalid: [
-    invalidCode("let foo = Components.classes['bar'];", "Components.classes", "Cc"),
-    invalidCode("let bar = Components.interfaces.bar;", "Components.interfaces", "Ci"),
-    invalidCode("Components.results.NS_ERROR_ILLEGAL_INPUT;", "Components.results", "Cr"),
-    invalidCode("Components.utils.reportError('fake');", "Components.utils", "Cu")
-  ]
+    invalidCode(
+      "let foo = Components.classes['bar'];",
+      "Components.classes",
+      "Cc"
+    ),
+    invalidCode(
+      "let bar = Components.interfaces.bar;",
+      "Components.interfaces",
+      "Ci"
+    ),
+    invalidCode(
+      "Components.results.NS_ERROR_ILLEGAL_INPUT;",
+      "Components.results",
+      "Cr"
+    ),
+    invalidCode(
+      "Components.utils.reportError('fake');",
+      "Components.utils",
+      "Cu"
+    ),
+  ],
 });
